@@ -1,11 +1,16 @@
 # Date::Simple - a simple date object
 
 package Date::Simple;
-
-$VERSION = '2.05';
+BEGIN {
+   $VERSION = '3.00';
+}
 use Exporter ();
 @ISA = ('Exporter');
-@EXPORT_OK = ('today', 'ymd', 'd8', 'date', 'leap_year', 'days_in_month');
+@EXPORT_OK = qw(
+                 today ymd d8 leap_year days_in_month
+                 date date_fmt date_d8 date_iso
+                 $Standard_Format
+               );
 %EXPORT_TAGS = ('all' => \@EXPORT_OK);
 
 # Try to load the C code.  If that fails, fall back to Date::Simple::NoXS.
@@ -36,7 +41,13 @@ use overload
     'ne'  => '_ne',
     'cmp' => '_compare',
     'bool' => sub { 1 },
-    '""'  => '_stringify';
+    '""'  => 'as_iso';
+
+use Scalar::Util qw(refaddr reftype);
+use warnings::register;
+require Date::Simple::Fmt;
+require Date::Simple::ISO;
+require Date::Simple::D8;
 
 sub today {
     my ($y, $m, $d) = (localtime) [5, 4, 3];
@@ -54,16 +65,15 @@ sub _inval {
 
 sub _new {
     my ($that, @ymd) = @_;
-    my ($class);
 
-    $class = ref ($that) || $that;
+    my $class = ref ($that) || $that;
 
-    if (scalar (@ymd) == 1) {
+    if ( @ymd == 1 ) {
 	my $x = $ymd[0];
-	if (ref ($x) eq 'ARRAY') {
+	if (ref $x and reftype($x) eq 'ARRAY') {
 	    @ymd = @$x;
 	}
-	elsif (UNIVERSAL::isa ($x, $class)) {
+	elsif (UNIVERSAL::isa ($x, __PACKAGE__)) {
 	    return ($x);
 	}
 	elsif ($x =~ /^(\d\d\d\d)-(\d\d)-(\d\d)$/
@@ -74,26 +84,39 @@ sub _new {
 	else {
 	    return (undef);
 	}
-    }
-    if (scalar (@ymd) == 0) {
-	return (today());
-    }
-    if (scalar (@ymd) == 3) {
+    }  # we fall through here...
+
+    # note we can end up here is they pass in [] as the date
+    return today() unless @ymd;
+
+    # to get here, we had one arg which was split,
+    # or 3 in the first place
+    if ( @ymd == 3 ) {
 	my $days = ymd_to_days (@ymd);
 	return undef if ! defined ($days);
 	return (bless (\$days, $class));
     }
-    _inval ($class, @ymd);
+
+    $class->_inval(@ymd);
 }
 
-sub date {
-    return (scalar (_new (__PACKAGE__, @_)));
+sub date { scalar __PACKAGE__->_new( @_ ) }
+
+sub date_fmt {
+    my $format=shift;
+    my $obj=Date::Simple::Fmt->_new(@_);
+    $obj->default_format($format)
+        if $obj;
+    $obj;
 }
 
+sub date_d8  { scalar Date::Simple::D8->_new(@_)  }
+sub date_iso { scalar Date::Simple::ISO->_new(@_) }
+
 # Same as date() but it's a method and croaks on error if called with
 # one arg.
 sub new {
-    my ($date);
+    my ($class,$date);
 
     $date = &_new;
     if (! $date && scalar (@_) == 1) {
@@ -112,18 +135,56 @@ sub _gmtime {
     return (0, 0, 0, $d, $m, $y);
 }
 
-sub format {
-    my ($self, $format) = @_;
-    return "$self" unless defined ($format);
-    require POSIX;
-    local $ENV{TZ} = 'UTC+0';
-    return POSIX::strftime ($format, _gmtime ($self));
+
+BEGIN {
+    our $Standard_Format="%Y-%m-%d";
+    my %fmts=( # Inside out parameter
+              'Date::Simple'      => $Standard_Format,
+              'Date::Simple::ISO' => $Standard_Format,
+              'Date::Simple::D8'  => "%Y%m%d",
+              'Date::Simple::Fmt' => $Standard_Format,
+             );
+
+    sub format {
+        my ($self,$format)=@_;
+
+        $format= $fmts{refaddr($self)||''} || $fmts{ref($self)} || $Standard_Format
+            if @_==1;
+
+        return "$self" unless defined ($format);
+        require POSIX;
+        local $ENV{TZ} = 'UTC+0';
+        return POSIX::strftime ($format, _gmtime ($self));
+    }
+
+    sub strftime { &format }
+    sub as_str   { &format }
+
+
+    sub default_format {
+        my ($self,$val)=@_;
+
+        my $o=refaddr($self) || $self;
+
+        if (@_>1) {
+            $fmts{$o}=$val;
+            warnings::warnif "Setting class specific date format '$o' to".
+            "'" .(defined $val ? $val : 'undef')."'"
+                unless ref $self;
+        }
+
+        return $fmts{$o}||$Standard_Format;
+    }
+
+    sub DESTROY {
+        delete $fmts{refaddr $_[0]};
+    }
 }
 
-sub strftime { &format }
-
 1;
 
+
+
 =head1 NAME
 
 Date::Simple - a simple date object
@@ -215,6 +276,10 @@ after today?
 
 What day of the week is a given date?
 
+=item Transparent date formatting.
+
+How should a date object be formatted.
+
 =back
 
 It does B<not> deal with hours, minutes, seconds, and time zones.
@@ -244,7 +309,46 @@ available on all platforms and are likely to change or disappear in
 future releases.  Please let the author know if you think any of them
 should be public.
 
-=cut
+=head2 Controlling output format.
+
+As of version 3.0 new ways of controlling the output formats of Date::Simple
+objects has been provided. However Date::Simple has traditionally provided
+few ways of stringification, a primary one via the format() method and another
+primary one via direct stringification. However the later is currently
+implemented as an XS routine and the former is implemented through a perl routine.
+This means that using format() is more expensive than stringification and
+that the stringification format is class specific.
+
+In order to alleviate some of these problems a new mechanism has been introduced
+to Date::Simple that allows for a per object level format default. In addition
+a set of utility classes that have different stringification overloads provided.
+These classes are simple subclasses of Date::Simple and beside the default format()
+and the overloaded stringification behaviour are identical to Date::Simple. In fact
+one is totally identical to Date::Simple and is provided mostly for completeness.
+
+The classes included are:
+
+=over 4
+
+=item Date::Simple::ISO
+
+Identical to Date::Simple in every respect but name.
+
+=item Date::Simple::D8
+
+Uses the D8 format (%Y%m%d) as the default format for printing. Uses XS for the
+overloaded stringification.
+
+=item Date::Simple::Fmt
+
+Uses the perl implemented format() as the default stringification mechanism. The first
+argument to the constructor is expected to be the format to use for the object.
+
+=back
+
+B<NOTE> its important to remember that the primary difference between the behaviour
+of objects of the different classes is how they are stringified when quoted, and what
+date format is used by default when the format() method is called. Nothing else differs.
 
 =head1 CONSTRUCTORS
 
@@ -273,6 +377,19 @@ functions, do C<use Date::Simple (':all');>.)  This function returns
 undef on all invalid input, rather than dying in some cases like
 C<new>.
 
+=item date_fmt (FMT,[ARG, ...])
+
+Equivelent to C<date> but creates a Date::Simple::Fmt object instead. The
+format is expected to be a valid POSIX::strftime format string.
+
+=item date_iso ([ARG, ...])
+
+Identical to C<date> but creates a Date::Simple::ISO object instead.
+
+=item date_D8 ([ARG, ...])
+
+Equivelent to C<date> but creates a Date::Simple::D8 object instead.
+
 =item today()
 
 Returns the current date according to C<localtime>.
@@ -318,7 +435,7 @@ Returns an object representing tomorrow.
 
 =item DATE->prev
 
-    my $yesterday = $today->prev;
+   my $yesterday = $today->prev;
 
 Returns an object representing yesterday.
 
@@ -356,13 +473,26 @@ Returns a list of three numbers: year, month, and day.
 Returns the "d8" representation (see C<d8>), like
 C<$date-E<gt>format("%Y%m%d")>.
 
-=item DATE->format (STRING)
+=item DATE->as_iso
 
-=item DATE->strftime (STRING)
+Returns the ISO 8601 representation of the date (eg '2004-01-01'),
+like C<$date-E<gt>format("%Y-%m-%d")>. This is in fact the default
+overloaded stringification mechanism and is provided mostly so
+other subclasses with different overloading can still do fast
+ISO style date output.
+
+=item DATE->as_str ([STRING])
+
+=item DATE->format ([STRING])
+
+=item DATE->strftime ([STRING])
 
 These functions are equivalent.  Return a string representing the
-date, in the format specified.  If you don't pass a parameter, an ISO
-8601 formatted date is returned.
+date, in the format specified.  If you don't pass a parameter, the default
+date format for the object is used if one has been specified, otherwise
+uses the default date format for the class the object is a member of, or as
+a last fallback uses the $Date::Simple::Standard_Format which is changeable,
+but probably shouldn't be modified. See C<default_format> for details.
 
     my $change_date = $date->format("%d %b %y");
     my $iso_date1 = $date->format("%Y-%m-%d");
@@ -373,6 +503,11 @@ strftime(3).  This is because we actually do pass it to strftime to
 format the date.  This may result in differing behavior across
 platforms and locales and may not even work everywhere.
 
+=item DATE->default_format ([FORMAT])
+
+This method sets or gets the default_format for the DATE object or class
+that it is called on.
+
 =back
 
 =head1 OPERATORS
@@ -423,7 +558,10 @@ equivalent to C<$date = $date + $number>.
 =item "$date"
 
 You can interpolate a date instance directly into a string, in the
-format specified by ISO 8601 (eg: 2000-01-17).
+format specified by ISO 8601 (eg: 2000-01-17) for Date::Simple and
+Date::Simple::ISO, for Date::Simple::D8 this is the same as calling
+as_d8() on the object, and for Date::Simple::Fmt this is the same as
+calling format() on the object.
 
 =back
 
@@ -441,15 +579,30 @@ Returns the number of days in MONTH, YEAR.
 
 =back
 
+=over 4
+
+=item leap_year (YEAR)
+
+Returns true if YEAR is a leap year.
+
+=item days_in_month (YEAR, MONTH)
+
+Returns the number of days in MONTH, YEAR.
+
+=back
+
+
 =head1 AUTHOR
 
     Marty Pauley <marty@kasei.com>
     John Tobey <jtobey@john-edwin-tobey.org>
+    Yves Orton <demerphq@hotmail.com>
 
 =head1 COPYRIGHT
 
-      Copyright (C) 2001  Kasei
+      Copyright (C) 2001  Kasei.
       Copyright (C) 2001,2002 John Tobey.
+      Copyright (C) 2004 Yves Orton.
 
       This program is free software; you can redistribute it and/or
       modify it under the terms of either:
@@ -467,4 +620,9 @@ Returns the number of days in MONTH, YEAR.
       but WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
+=head1 SEE ALSO
+
+L<Date::Simple::Fmt> L<Date::Simple::ISO> L<Date::Simple::D8>
+and of course L<perl>
+
 =cut
